      REAL*8 FUNCTION DNORDF(X)
C
C     Evaluates the standard normal (Gaussian) distribution function.
C     This function is made to replace the IMSL function DNORDF when the
C     IMSL routine is not available.  All routines in this file are based
C     on those contained in the book "Numerical Recipes, The Art of 
C     Scientific Computing" by William H. Press, Brian P. Flannery, 
C     Saul A. Teukolsky, and William T. Vetterling (Cambridge University
C     Press, Cambridge, 1987).
C
      IMPLICIT NONE
      REAL*8 X,ERFC
C
C     Based on the value of X, various methods will be used to evaluate
C     this function.  This is also done in order to optimize the run
C     time of the function if evaluated many sigma widths from the center.
C
      IF(X.GT.8.D0) THEN
        DNORDF = 1.0
      ELSE IF(X.LT.-12.D0) THEN
        DNORDF = 0.0
      ELSE
C
C       Do evaluation using the complementary error function with double
C       precision numbers.
C
        DNORDF = ERFC(-X/DSQRT(2.D0))/2.D0
      END IF
C
      RETURN
      END
C
C
      REAL FUNCTION ANORDF(X)
C
C     Conversion function to the single precission ANORDF format
C
      IMPLICIT NONE
      REAL X
      REAL*8 DNORDF
      EXTERNAL DNORDF
C
      ANORDF = SNGL(DNORDF(DBLE(X)))
C
      RETURN
      END
C
C
      REAL*8 FUNCTION ERFC(X)
C
C     Evaluates the complementary error function.  The function
C     uses the incomplete gamma functions to do the evaluation.
C     Function uses double precision numbers.
C
      IMPLICIT NONE
      REAL*8 X,GAMMP,GAMMQ
C
      IF(X.LT.0.D0) THEN
        ERFC = 1. + GAMMP(0.5D0,X**2.)
      ELSE
        ERFC = GAMMQ(0.5D0,X**2.)
      END IF
C
      RETURN
      END
C
C
      REAL*8 FUNCTION ERF(X)
C
C     Evaluates the error function.  The function uses the 
C     incomplete gamma functions to do the evaluation.
C     Function uses double precision numbers.
C
      IMPLICIT NONE
      REAL*8 X,GAMMP
C
      IF(X.LT.0.D0) THEN
        ERF = -GAMMP(0.5D0,X**2.)
      ELSE
        ERF = GAMMP(0.5D0,X**2.)
      END IF
C
      RETURN
      END
C
C
      REAL*8 FUNCTION GAMMP(A,X)
C
C     Evaluates the incomplete gamma function P(a,x).
C     Function uses double precision numbers.
C
      IMPLICIT NONE
      REAL*8 A,X,GAMSER,GLN,GAMMCF
C
      IF(X.LT.0.D0 .OR. A.LE.0.D0) PAUSE 'Improper value in GAMMP'
      IF(X.LT.A+1.D0 .OR. (A.LT.1.D0 .AND. X.LT.A+10)) THEN
        CALL GSER(GAMSER,A,X,GLN)  ! Use the series representation.
        GAMMP = GAMSER
      ELSE
        CALL GCF(GAMMCF,A,X,GLN)   ! Use the continued fraction representation
        GAMMP = 1.D0 - GAMMCF      ! and take its compliment.
      END IF
C
      RETURN
      END
C
C
      REAL*8 FUNCTION GAMMQ(A,X)
C
C     Evaluates the incomplete gamma function Q(a,x) = 1 - P(a,x).
C     Function uses double precision numbers.
C
      IMPLICIT NONE
      REAL*8 A,X,GAMSER,GLN,GAMMCF
C
      IF(X.LT.0.D0 .OR. A.LE.0.D0) PAUSE 'Improper value in GAMMQ'
      IF(X.LT.A+1.D0 .OR. (A.LT.1.D0 .AND. X.LT.A+10)) THEN
        CALL GSER(GAMSER,A,X,GLN)  ! Use the series representation
        GAMMQ = 1.D0 - GAMSER      ! and take its compliment.
      ELSE
        CALL GCF(GAMMCF,A,X,GLN)   ! Use the continued fraction representation.
        GAMMQ = GAMMCF
      END IF
C
      RETURN
      END
C
C
      SUBROUTINE GSER(GAMSER,A,X,GLN)
C
C     Evaluates the incomplete gamma function P(a,x) by its series
C     representation and returns the value as GAMSER.  Also evaluates
C     ln(Gamma(a)) and returns as GLN.  Work with double precision numbers.
C
      IMPLICIT NONE
      REAL*8 GAMSER,A,X,GLN,EPS,GAMMLN,AP,SUM,DEL
      INTEGER N,ITMAX
      PARAMETER (ITMAX=100,EPS=1.D-10)
C
      GLN = GAMMLN(A)
      IF(X.LE.0.D0) THEN
        IF(X.LT.0.D0) PAUSE 'X.LT.0. in GSER'
        GAMSER = 0.D0
        RETURN
      END IF
      AP = A
      SUM = 1.D0/A
      DEL = SUM
      N = 1
      DO WHILE (DABS(DEL).GE.DABS(SUM)*EPS .AND. N.LT.ITMAX)
        N = N + 1
        AP = AP + 1.D0
        DEL = DEL*X/AP
        SUM = SUM + DEL
      END DO
      IF(N.GE.ITMAX) PAUSE 'A too large, ITMAX too small in GSER'
      GAMSER = SUM*DEXP(-X + A*DLOG(X) - GLN)
C
      RETURN
      END
C
C
      SUBROUTINE GCF(GAMMCF,A,X,GLN)
C
C     Evaluates the incomplete gamma function Q(a,x) its by continued fraction
C     representation and returns the value as GAMMCF.  Also evaluates
C     ln(Gamma(a)) and returns as GLN.  Will use double precision numbers for
C     this evaluation.  Work with double precision numbers.
C
      IMPLICIT NONE
      REAL*8 GAMMCF,A,X,GLN,GAMMLN,EPS,GOLD,A0,A1,B0,B1,FAC,AN,ANA,ANF,G
      INTEGER N,ITMAX
      PARAMETER (ITMAX=100,EPS=1.D-15)
C
      GLN = GAMMLN(A)
      GOLD = 0.D0               ! This is the previous value tested for 
      A0 = 1.D0                 ! convergence.  Set up the coefficients 
      A1 = X                    ! for evaluating the continued fraction.
      B0 = 0.D0                 ! FAC is the renormaization factor for
      B1 = 1.D0                 ! preventing overflow of the partial
      FAC = 1.D0                ! numerators adn denominators.
      DO N = 1,ITMAX
        AN = DBLE(N)
        ANA = AN - A
        A0 = (A1 + A0*ANA)*FAC  ! One step of the recurrence formula.
        B0 = (B1 + B0*ANA)*FAC
        ANF = AN*FAC
        A1 = X*A0 + ANF*A1      ! Next step of the recurrence formula.
        B1 = X*B0 + ANF*B1
        IF(A1.NE.0.D0) THEN     ! Renormalize?  If yes, reset FAC and 
          FAC = 1.D0/A1         ! obtain new value of answer.  Check for
          G = B1*FAC            ! convergence.  If converged, then exit.
          IF(DABS(G-GOLD)/G.LT.EPS) THEN
            GAMMCF = DEXP(-X + A*DLOG(X) - GLN)*G  ! Put factors in front.
            RETURN
          END IF
          GOLD = G              ! Not converged, save new value.
        END IF
      END DO
      PAUSE 'A too large, ITMAX too small in GCF'
      GAMMCF = DEXP(-X + A*DLOG(X) - GLN)*G
C
      RETURN
      END
C
C
      REAL*8 FUNCTION GAMMLN(X)
C
C     Evaluates ln(Gamma(X)) for X>0.  Full accuracy is obtained for
C     X>1.  For 0<X<1, a reflection formula will be used.  All internal
C     arithmetic will be performed using double precision.
C
      IMPLICIT NONE
      INTEGER J
      REAL*8 X,COF(6),PI,STP,Y,Z,TMP,SER
      DATA COF/76.18009173D0,-86.50532033D0,24.01409822D0,
     *  -1.231739516D0,.120858003D-2,-.536382D-5/
C
C     For 0<X<1, set up for the recurrence formula and evaluate 
C     ln(Gamma(1+Z).
C
      PI = DACOS(-1.D0)
      STP = DSQRT(2.D0*PI)
      IF(X.EQ.1.D0) THEN
        GAMMLN = 0.D0
        RETURN
      ELSE IF(X.LT.1.D0) THEN
        IF(X.EQ.0.5D0) THEN
          GAMMLN = SNGL(DLOG(DSQRT(PI)))
          RETURN
        END IF
        Z = 1.D0 - X
        Y = Z
      ELSE
        Y = X - 1.D0
      END IF
      TMP = (Y + 0.5D0)*DLOG(Y + 5.5D0) - (Y + 5.5D0)
      SER = 1.D0
      DO J=1,6
        Y = Y + 1.D0
        SER = SER + COF(J)/Y
      END DO
      GAMMLN = TMP + DLOG(STP*SER)
      IF(X.LT.1.D0) THEN             !Use recurrence formula.
        GAMMLN = DLOG(PI*Z) - DLOG(DSIN(PI*Z)) - GAMMLN
      END IF
C
      RETURN
      END

